using System.Collections.Generic;
using Siger.Middlelayer.Common.ModuleEnum;
using Siger.Middlelayer.Repository.Data;
using Siger.Middlelayer.Repository.Entities;
using Siger.Middlelayer.Repository.Response;
using Siger.Middlelayer.Utility.ImportEntities;

namespace Siger.Middlelayer.Repository.Repositories.Interface
{
    public interface Isiger_wms_bussinese_contactsRepository: IApiConfigRepositoryBase<siger_wms_bussinese_contacts>
    {
        IEnumerable<ResponseGetBusinessList> GetBusinessList(int materialID, int pid,string name, WMSEnum type, int typenameid, out int count, int page = 1, int pageSize = 10);
        IEnumerable<ResponseGetBusinessList> GetExportData(int pid,string name, WMSEnum type, int typenameid);

        IEnumerable<IdTitle> GetPnSupplierIds(int projectId, int materialId);

        IEnumerable<ResponseIdText> GetSupplierList(int projectId);

        IEnumerable<ResponseIdText> GetSupplierData(int projectId);

        CommonImportResult ImportBusiness(IEnumerable<wmsbusinessinfo> data, WMSEnum type, int projectid, int userId);
    }
}
